/*
 * $Id: //trid/release/pam_otp_auth-3.2.2/extern.h#1 $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 *  For alternative licensing, contact licensing@tri-dsystems.com.
 *
 * Copyright 2005,2006 TRI-D Systems, Inc.
 */

#ifndef EXTERN_H
#define EXTERN_H

#include "ident.h"
RCSIDH(extern_h, "$Id: //trid/release/pam_otp_auth-3.2.2/extern.h#1 $")

#include <sys/types.h>
#include <pthread.h>

/* otpd rendezvous point */
#define OTP_OTPD_RP "/var/run/otpd/socket"

/* Default prompt for presentation of challenge */
#define OTP_CHALLENGE_PROMPT "Challenge: %s\n Response: "

/* Password that means "challenge me" in fast_sync mode */
#define OTP_CHALLENGE_REQ "challenge"

/* Default prompt in fast_sync mode */
#define OTP_FAST_PROMPT "Password: "


/*
 * You shouldn't change anything past this point
 */

/* struct used for instance/option data */
typedef struct otp_option_t {
  char *otpd_rp;	/* otpd rendezvous point                           */
  char *fast_prompt;	/* fast mode prompt                                */
  char *chal_prompt;	/* text to present challenge to user, must have %s */
  int chal_len;	        /* challenge length, min 5 digits                  */
  int fast_sync;	/* response-before-challenge mode                  */
  int allow_sync;	/* useful to override pwdfile card_type settings   */
  int allow_async;	/* C/R mode allowed?                               */
  char *chal_req;	/* keyword requesting challenge for fast_sync mode */
  int useauthtok;	/* use_first_pass (undocumented)                   */

  int debug;
} otp_option_t;

/* otp_pw_valid.c */
extern int otp_pw_valid(const char *, const char *, const char *,
                        const otp_option_t *);

/* otp_support.c */
#include <security/pam_appl.h>
extern int otp_parse_opts(int, const char **, const char *, otp_option_t *);
extern void otp_free_opts(otp_option_t *);
extern int otp_interact(pam_handle_t *, char *, int, char **);

/* otp_util.c */
extern int otp_async_challenge(char * /* [OTP_MAX_CHALLENGE_LEN+1] */, int);
extern int _otp_read(int, char *, size_t, const char *);
extern int _otp_write(int, const char *, size_t, const char *);

#define otp_read(a, b, c) _otp_read((a), (b), (c), __func__)
#define otp_write(a, b, c) _otp_write((a), (b), (c), __func__)

#endif /* EXTERN_H */
