/*
 * $Id: //trid/release/pam_otp_auth-3.2.2/otp_util.c#1 $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 *  For alternative licensing, contact licensing@tri-dsystems.com.
 *
 * Copyright 2001,2002  Google, Inc.
 * Copyright 2005,2006 TRI-D Systems, Inc.
 */

#include "ident.h"
RCSID("$Id: //trid/release/pam_otp_auth-3.2.2/otp_util.c#1 $")

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>

#include "extern.h"
#include "otp.h"

/*
 * Return a random challenge.
 * challenge is filled in on successful return (must be at least size len+1).
 * Returns 0 on success, -1 on failure.
 * NOTE: This is really cryptocard-specific (automatic ASCII conversion
 * and null termination).
 */
int
otp_async_challenge(char challenge[OTP_MAX_CHALLENGE_LEN + 1], int len)
{
  unsigned char rawchallenge[OTP_MAX_CHALLENGE_LEN];
  int i, fd;

  if ((fd = open("/dev/urandom", O_RDONLY)) == -1) {
    syslog(LOG_ERR, "pam_otp_auth: %s: error opening %s: %s",
           __func__, "/dev/urandom", strerror(errno));
    return -1;
  }

  if (otp_read(fd, (char *) rawchallenge, len) == -1) {
    syslog(LOG_ERR, "pam_otp_auth: %s: failed to obtain random data", __func__);
    return -1;
  }
  /* Convert the raw bytes to ASCII decimal. */
  for (i = 0; i < len; ++i)
    challenge[i] = '0' + rawchallenge[i] % 10;
  challenge[len] = '\0';

  (void) close(fd);
  return 0;
}

/*
 * Full read with logging, and close on failure.
 * Returns nread on success, -1 on failures.
 */
int
_otp_read(int fd, char *buf, size_t len, const char *caller)
{
  ssize_t n;
  size_t nread = 0;	/* bytes read into buf */

  while (nread < len) {
    if ((n = read(fd, &buf[nread], len - nread)) == -1) {
      if (errno == EINTR) {
        continue;
      } else {
        syslog(LOG_ERR, "pam_otp_auth: %s: read: %s", caller, strerror(errno));
        (void) close(fd);
        return -1;
      }
    }
    if (!n) {
      syslog(LOG_ERR, "pam_otp_auth: %s: disconnect", caller);
      (void) close(fd);
      return -1;
    }
    nread += n;
  } /* while (more to read) */

  return nread;
}

/*
 * Full write with logging, and close on failure.
 * Returns 0 on success, -1 on failure.
 */
int
_otp_write(int fd, const char *buf, size_t len, const char *caller)
{
  size_t nleft = len;
  ssize_t nwrote;

  while (nleft) {
    if ((nwrote = write(fd, &buf[len - nleft], nleft)) == -1) {
      if (errno == EINTR || errno == EPIPE) {
        continue;
      } else {
        syslog(LOG_ERR, "pam_otp_auth: %s: write: %s", caller, strerror(errno));
        (void) close(fd);
        return -1;
      }
    }
    nleft -= nwrote;
  }

  return 0;
}
